// Nano  - 328P old boot legger - AVRISP mkII
// used for the small clock  with 3 volt led bars and driving direct from nano
// this is the current version for the double sides pcb 140 x 140 3v filament clock
// this version includes the alarm clock function
// this version has separate five second flashing leds round the edge
// built 18/9/2021
// ATMEGA328P CH340G
// v2
// set up for blue lights



#include <Wire.h>
#include "RTClib.h"
#include <EEPROM.h>
RTC_DS3231 rtc;

//led array 0 to 25;
byte barOneToTwentyFour[25] = {
  B11000000, B11100000, B11010000, B11001000, B11000100,
  B11000010, B11000001, B11000001, B11000010, B11000100,
  B11001000, B11010000, B11100000, B11100000, B11010000,
  B11001000, B11000100, B11000010, B11000001, B11000001,
  B11000010, B11000100, B11001000, B11010000, B11100000
};
// single minute array
byte barSingleMinute[5] = {
  B11000000, B11100000, B11101000, B11101100, B11111100
};

// This is the brightness control array for the hour and minute bars;

word barBrightnessControl [10] = {
  0, 20, 40, 80, 110, 150, 250, 300, 400, 500
};

//brightness control array for the central minute white leds;

word whiteBrightnessControl [10] = {
  1, 1, 1, 1, 1, 1, 1, 7, 8, 9
};

// brightness control array for the 12 outer leds;

word outerBrightnessControl [10] = {
  0, 2, 4, 5, 6, 7, 8, 0, 0, 0
};

int minute;
int hour;
int second;
byte BlueOne = 0;
byte BlueTwo = 0;
byte BlueThree = 0;
byte BlueFour = 0;

byte countHour;               		// used in hour multiplex routine
byte countHour1 = 0;          	// used in clock setup
byte countMinute = 0;         	// used in minute multiplex routine
byte countMinute1;           	 // stores clock minutes
byte singleMinute;            		// stores the centre minutes
byte singleMinute1;           	// temp store to transfer the single minutes
byte countHour2;              		// stores the real clock time for the alarm system
byte countSecond;             	// stores clock seconds for blue lights
byte countSecond1;            	// stores true clock seconds for blue lights
byte countSecond2;            	// second store for true clock seconds
byte singleSecond;            		// stores clock seconds for blue lights
byte singleSecond2 = 0;       	// stores flashing seconds for blue lights
byte alarmOnOff = 0;          	// for pulsing the alarm
byte ledControlOne = 3;       	// controls the display brightness of bars, minute leds, outer leds default on powerup
boolean buttonTest = 0;       	// stores the result of each button pressed
boolean dimDisplay ;          	// on 1 the display is dimmed
boolean alarmIsOn = 1;        	// lights the alarm centre light when at 1
boolean alarmFlag;            	// togals alarm on and off when alarm button is pressed
boolean testFlag1 = 0;        	// used in alarm to display serial information of set up
boolean testFlag2 = 0;       	 // used in alarm to display serial information of set up
boolean testFlag3 = 0;        	// used in alarm to display serial information of set up
boolean silenceAlarm = 0;     	// if alarm is running any button will silence it
boolean singleSecond1 = 0;    	// on off for blue lights
boolean flag9  = 0 ;          		// used to switch the 4 minute centre display on
boolean lightStep = 0;        	// switch the light stepping on
boolean singleSecond3 = 0;    	// stores flashing seconds for blue lights
int alarmMinutes;             		// store for minute in alarm
int alarmHours;               		// store for hours in alarm
word secondTimer = 0;         	// used in blue led display of seconds
word ledControlTwo;           	// final bar time delay for brightness
word ledControlThree;        	 // final leds time delay for brightness
word ledControlFour;          	// centre blue led  time delay for brightness
word ledControlFive;          	// outer number leds time delay for brightness



//------------- Setup from here -----------------------------------------------------------------------------

void setup() {

  Wire.begin();
#ifndef ESP8266
  while (!Serial);            				// for Leonardo/Micro/Zero
#endif
  Serial.begin (9600);
  delay(3000);                				// wait for console opening
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }
  //     rtc.adjust(DateTime(2014, 1, 21, 14, 02, 00));  // used only to test clock after construction
  pinMode(A3, OUTPUT);        			// alarm bleeper
  pinMode(A2, OUTPUT);        			// blue 1 - 6 leds
  pinMode(A0, INPUT_PULLUP);  		// Hour button
  pinMode(A1, INPUT_PULLUP);  		// Minutes button
  pinMode(A6, INPUT);         			// alarm set button
  pinMode(A7, INPUT);        			 // Dim button
  pinMode(2, OUTPUT);         			//  blue 7 to 12 leds
  pinMode(3, OUTPUT);         			//  inner 7 to 12
  pinMode(4, OUTPUT);         			//  inner 1 to 6
  pinMode(5, OUTPUT);         			//  outer 1 to 6
  pinMode(6, OUTPUT);         			//  outer 7 to 12
  pinMode(7, OUTPUT);         			//  white lights
  DDRB = B11111111;           			//  sets 8 to 13 to outputs and direct drives to filaments
  PORTB = B11000000;          			//  clear pin outputs that drive the leds.

  // all below are setting defaults on power uo
  digitalWrite(2, HIGH);       			//   blue 7 to 12
  digitalWrite(3, HIGH);       			//  inner 7 to 12
  digitalWrite(4, HIGH);       			//  inner 1 to 6
  digitalWrite(5, HIGH);       			//  outer 1 to 6
  digitalWrite(6, HIGH);       			//  outer 7 to 12
  digitalWrite(7, HIGH);       			//  white single minute leds
  digitalWrite(A2, HIGH);      			//  blue 1 - 6
  digitalWrite(A3, HIGH);   			//  beep alarm on startup
  delay (4);
  digitalWrite(A3, LOW);       			//   switch alarm off


  buttonTest = analogRead(A6) > 100 ? 1 : 0;
  Serial.print("Left button A7 =  ");
  Serial.print(buttonTest);
  Serial.println();
  buttonTest = digitalRead (A0);
  Serial.print("Left middle A0 =  ");
  Serial.print(buttonTest);
  Serial.println();
  buttonTest = digitalRead(A1);
  Serial.print("Right middle A1 =  ");
  Serial.print(buttonTest);
  Serial.println();
  buttonTest = analogRead(A7) > 100 ? 1 : 0;
  Serial.print("Right button A7 =  ");
  Serial.print(buttonTest);
  Serial.println();



  // EEprom store all these settings override initialisation

  EEPROM.get(0, (dimDisplay));    			// store for the leds dim or bright
  EEPROM.get(1, (alarmMinutes)); 			// store for the alarm minutes
  EEPROM.get(3, (alarmHours)); 				 // store for the alarm hours
  EEPROM.get(2, (alarmIsOn)); 				 // store for the alarm hours
  //
  // store backed up brightness

  EEPROM.get(4, (ledControlOne)); 			// store for the alarm hours
  ledControlTwo = barBrightnessControl[ledControlOne];
  ledControlThree = whiteBrightnessControl[ledControlOne];
  ledControlFour = outerBrightnessControl[ledControlOne];

}
//------------------------ Start of the programme-------------------------------

void loop() {

  if (testFlag2 == 0)                           	// if alarm is running isolate all buttons
  {
    buttonTest = digitalRead (A0);             	 // left centre button for dim lights
    if (buttonTest == 0)                       	 // test if pressed
      BrightnessControl();                      	// if pressed go to dim lights
    buttonTest = digitalRead(A1);              	 // right centre button for alarm
    if (buttonTest == 0)                       	 // test if pressed
      alarmClock ();                           		 // if pressed got to arm setup
    buttonTest = analogRead(A6) > 100 ? 1 : 0;  // left button for hours
    if (buttonTest == 0)                        	// test if pressed
      hourIncrument();                         	 // if pressed go to hour setup
    buttonTest = analogRead(A7) > 100 ? 1 : 0;  // right button for minutes
    if (buttonTest == 0)                        	// test if pressed
      minuteIncrument();                       	 // if pressed go to minutes setup
  }
  else
  {
    buttonTest = digitalRead (A0);               // left centre button for dim lights
    if (buttonTest == 0)                         	// test if pressed
      silenceAlarm = 1;                         	 // any button will silence alarm
    buttonTest = digitalRead(A1);                // right centre button for alarm
    if (buttonTest == 0)                         	// test if pressed
      silenceAlarm = 1;                          	// any button will silence alarm
    buttonTest = analogRead(A6) > 100 ? 1 : 0;   // left button for hours
    if (buttonTest == 0)                        	 // test if pressed
      silenceAlarm = 1;                          	// any button will silence alarm
    buttonTest = analogRead(A7) > 100 ? 1 : 0;   // right button for minutes
    if (buttonTest == 0)                         	// test if pressed
      silenceAlarm = 1;                          	// any button will silence alarm
  }
  if (alarmIsOn == 1)                           	 // test if alarm is set
    checkTheAlarm ();                          	 // if set go to the alarm sounding routine


  // from here are the multiplex routines for the display

  SetTime();                                  		// takes current time from tiny clock module
  DisplayMultiplexSetup();                    	// set up the multiplex against current time

  if (ledControlOne >> 1)
  {
    oneToTwelveBlueLightControl ();                // multiplex blue lights 1 - 12 & single minute
  }
  else
  {
    SingleMinutedisplay();                       	// multiplex of single centre minutes
  }
  MinuteDisplay();                            	// multiplex of minute hand

  if (countMinute  == countHour)              	// don't light hour if same position of minute hand
    delay (1);                			 // just pretend that the multiplex hour display has run
  else
    HourDisplay();                            		// multiplex of hour hand

} // end of void of main loop

//-------------------------- end of normal running subroutines from here -----------------------------

//______________________________ alarm sounding routine _____________________________-

void checkTheAlarm ()
{
  DateTime now = rtc.now();
  countHour1 = (now.hour());                      // store hours
  countMinute1 = (now.minute());                  // store minutes


  if ( alarmHours == countHour1)                 // check for hour match
  {
    if (testFlag1 == 0)                          	// test flag1 goes to 1 when there is a match
    {
      Serial.print("alarm hour match =  ");
      Serial.print(countHour1);
      Serial.println();
      testFlag1 = 1;

    }
    if (alarmMinutes == countMinute1)           // check for minute match
    {
      alarmOnOff += 1;
      if (alarmOnOff >= 100)                    // adjust for longer delay between bleeps
      {
        if (silenceAlarm == 0)
        {
          digitalWrite(A3, HIGH);               	// sound bleeper
          delay (100);                          		// adjust for long bleep time
          digitalWrite(A3, LOW);
          alarmOnOff = 0;
        }
      }
      if (testFlag2 == 0)                      	// test flag2 goes to 1 when there is a match
      {
        Serial.print("alarm minutes match =  ");
        Serial.print(countMinute1);
        Serial.println();
        testFlag2 = 1;
        testFlag3 = 1;                         		// open up test flag 3 when no longer a match
      }
    }
    else
    {
      testFlag2 = 0;                         		 // with no match the test flag 2 remains at 0
      if (testFlag3 == 1)
      {
        Serial.print("alarm minutes are no match =  ");
        Serial.print(countMinute1);
        Serial.println();
        testFlag3 = 0;
        if (silenceAlarm == 1)
        {
          silenceAlarm = 0;
          Serial.print("alarm out of silence mode =  ");
          Serial.print(silenceAlarm);
          Serial.println();
        }
      }
    }
  }
  else
  {
    testFlag1 = 0;
  }
} //end of void check the alarm

// _________________________ alarm clock routines ______________________________________

void alarmClock ()
{
  Serial.print("alarmbutton pressed =  ");
  Serial.print(buttonTest);
  Serial.println();


  boolean incMinute;
  boolean incHour;
  int timerCount = 0;      				// used in this routine to time out no button press
  while (buttonTest == 0)  				// stay in loop until button is released
  {
    buttonTest = digitalRead(A1);                              // right centre button for alarm
    countMinute1 = alarmMinutes;                               // read stored alarm minutes
    countMinute = ((countMinute1 / 5));                       // bring the minutes down to 5 minute steps
    singleMinute1 = ( (countMinute) * 5);                     // time without the extra minutes
    singleMinute = ( (countMinute1) - (singleMinute1));       	// just the extra minutes
    if (countMinute == 0)                                     		// change a zero result to be  twelve o clock
      countMinute = 12;
    countHour = alarmHours;
  }

  // switch alarm on and off

  if (alarmFlag == 0)                                        	 // for alarm on or off
  {
    alarmIsOn = 1;                                           	 // alarm is on
    alarmFlag = 1;
  }
  else
  {
    alarmIsOn = 0;                                            	// alarm is off
    alarmFlag = 0;
  }

  // display current alarm time

  while (timerCount < 300)
  {
    timerCount = timerCount + 1;                                  // start timer increment
    SingleMinutedisplay();                                        	// multiplex of single centre minutes
    MinuteDisplay();                                             	 // multiplex of minute hand
    HourDisplay();                                               	 // multiplex of hour hand
    delay (10);

    // check minute increment button

    incMinute = analogRead(A7) > 100 ? 1 : 0;                   	 // right button for minutes
    if (incMinute == 0)                                         		// if button pressed increment one minute
    {
      alarmIsOn = 1;                                            		// alarm is on
      alarmFlag = 1;
      timerCount = 100;                                         		// restart timer count but less time
      countMinute1 += 1;
      if (countMinute1 >= 60)                                   		// if increment exceeds 60 reset to 0
        countMinute1 = 0;
      countMinute = ((countMinute1 / 5));                       	// bring the minutes down to 5 minute steps
      singleMinute1 = ( (countMinute) * 5);                    	 // time without the extra minutes
      singleMinute = ( (countMinute1) - (singleMinute1));       // just the extra minutes
      if (countMinute == 0)                                     		// change a zero result to be  twelve o clock
        countMinute = 12;
    }
    while (incMinute == 0)                                      		// loop until button is released
    {
      incMinute = analogRead(A7) > 100 ? 1 : 0;                 	// right button for minutes
      delay (10);                                              			 // bounce delay
    }

    // check hour increment button

    incHour = analogRead(A6) > 100 ? 1 : 0;                     	// left button for hours
    if (incHour == 0)                                           		// if button pressed increment one hour
    {
      alarmIsOn = 1;                                            		// alarm is on
      alarmFlag = 1;
      timerCount = 100;                                         		// restart timer count but less time
      countHour += 1;
      if ((countHour) == 25)                                    		// in twenty four hour mode at 25 count take back to one
        countHour = 1;
      timerCount = 200;                                         		// reduce the timer timeout if no button pressed
    }
    while (incHour == 0)                                        		// loop until button is released
    {
      incHour = analogRead(A6) > 100 ? 1 : 0;                   	// left button for hours
      delay (10);                                               			// bounce delay
    }
  }
  alarmHours = countHour;
  EEPROM.write(3, (alarmHours));                                	// store hours in alarm on power down
  alarmMinutes = countMinute1;                                  	// store new minutes for alarm on power down
  EEPROM.write(1, (alarmMinutes));
  EEPROM.write(2, (alarmIsOn));                                	 // store how alarm is set on power down
  Serial.print("alarm minutes =  ");
  Serial.print(alarmMinutes);
  Serial.println();
  Serial.print("alarm Hours =  ");
  Serial.print(alarmHours);
  Serial.println();
  testFlag1 = 0;
  testFlag2 = 0;
  countMinute1 = 61;
  countHour = 0;
  digitalWrite(A3, HIGH);                                       //  bleep when alarm time is set
  delay (5);
  digitalWrite(A3, LOW);

} // end of void alarm clock

// ___________________________ set up routine for the minutes __________________________

void minuteIncrument()
{
  int timerCount = 0;                          	 // used in this routine to time out no button press
  int longTimer = 0;                           	 // if button is kept pressed count five minute steps
  int longTimer1 = 0;                           	// delay to detect button held down
  boolean stepFlag1 = 0;                        	// used if button held down to default to twelve to block timer
  DateTime now = rtc.now();
  countHour1 = (now.hour());                   	// store current hours before setting up minutes
  Serial.print("countHour1 at start of minute adjust =  ");
  Serial.print(countHour1);
  Serial.println();

  while (buttonTest == 0)                       		// stay in loop until button is released
  {
    buttonTest = analogRead(A7) > 100 ? 1 : 0;  	// right button for minutes
    if (stepFlag1 == 0)                         		// allow long timer to run
      longTimer  += 1;
    if (longTimer >= 2000)
    {
      longTimer1 += 1;
      longTimer = 0;
    }
    if (longTimer1 >= 10)                                      	// after delay go to default twelve o clock
    {
      countMinute1 = 0;
      countMinute = 12;
      MinuteDisplay();                                         	// multiplex of minute hand
      stepFlag1 = 1;                                         		 // 12.00 set now switch off timer
      delay (5);
    }
  }
  while (timerCount < 500)
  {
    timerCount = timerCount + 1;                              	// start timer increment
    SingleMinutedisplay();                                    	// multiplex of single centre minutes
    MinuteDisplay();                                         	 // multiplex of minute hand
    buttonTest = analogRead(A7) > 100 ? 1 : 0;            // right button for minutes
    if (buttonTest == 0)                                      	// if button pressed increment one minute
    {
      countMinute1 += 1;
      if (countMinute1 == 61)                                 	// if increment exceeds 60 reset to 1
        countMinute1 = 1;
      countMinute = ((countMinute1 / 5));                     // bring the minutes down to 5 minute steps
      singleMinute1 = ( (countMinute) * 5);                   // time without the extra minutes
      singleMinute = ( (countMinute1) - (singleMinute1));     // just the extra minutes
      if (countMinute == 0)                                  	 // change a zero result to be  twelve o clock
        countMinute = 12;

      timerCount = 150;                                       	// reduce the timer timeout if no button pressed
    }
    while (buttonTest == 0)                                   	// loop until button is released
    {
      buttonTest = analogRead(A7) > 100 ? 1 : 0;                         // right button for minutes
      delay (10);
      rtc.adjust(DateTime(2014, 1, 21, (countHour1), countMinute1, 00)); // set new minutes
    }
    delay (10);                                                         			// bounce delay
  }  // end of timer loop to get out
  rtc.adjust(DateTime(2014, 1, 21, (countHour1), countMinute1, 00));    // set new minutes
  digitalWrite(A3, HIGH);                                    			 //  bleep when time is set
  delay (5);
  digitalWrite(A3, LOW);

}  // end of void minute increment

// ________________________ set up routine for the hour in twenty four hour mode _________

void hourIncrument()
{
  int timerCount = 0;                                                 		// used in this routine to time out no button press
  DateTime now = rtc.now();
  countHour = (now.hour());                                          	 // store current hours before setting up hours
  while (buttonTest == 0)                                             	// stay in loop until button is released
  {
    buttonTest = analogRead(A6) > 100 ? 1 : 0;                        // left button for hours
    delay (10);
  }
  while (timerCount < 500)
  {
    timerCount = timerCount + 1;                  			// start timer increment
    HourDisplay();                               			 // display just the hour in this loop
    delay (10);
    buttonTest = analogRead(A6) > 100 ? 1 : 0;    		// left button for hours
    if (buttonTest == 0)                          			// if button pressed increment one hour
    {
      countHour += 1;
      if ((countHour) == 25)                      			// in twenty four hour mode at 25 count take back to one
        countHour = 1;
      timerCount = 200;                           			// reduce the timer timeout if no button pressed
    }
    while (buttonTest == 0)                       			// loop until button is released
    {
      buttonTest = analogRead(A6) > 100 ? 1 : 0;  		// left button for hours
      delay (10);                                				// bounce delay
    }

  }
  rtc.adjust(DateTime(2014, 1, 21, (countHour), countMinute1, 00)); 	// set new hour
  digitalWrite(A3, HIGH);                        				//  bleep when time is set
  delay (5);
  digitalWrite(A3, LOW);

}  // end of void for hour increment

//------------------ Clock routine----------------------------

void SetTime()
{
  DateTime now = rtc.now();
  countHour1 = (now.hour());         				 // store hours
  countMinute1 = (now.minute());      			// store minutes
  countSecond1 = (now.second());      			// store seconds
  countHour2 = countHour1;
  if ((countHour1) == 0)              				// in 24 hour mode the 12 midnight is 0 change to display correctly
    countHour1 = 12;                  				// this does not effect the alarm that needs the 0 reading
  if (countHour1 >= 13)
    countHour1 = (countHour1 - 12);

  if ((countMinute1) >= 35)
  {
    countHour = (countHour1);
    countHour += 1;
    if ((countHour) == 13)
      countHour = 1;
  }
  else
    countHour = (countHour1);
}  // void end of settime

// -------------------- multiplex of display routines from here ------

// -------------------- Set Up display multiplex routines ------------

void DisplayMultiplexSetup () {

  countMinute = ((countMinute1 / 5));                       		// bring the minutes down to 5 minute steps
  singleMinute1 = ( (countMinute) * 5);                    		 // time without the extra minutes
  singleMinute = ( (countMinute1) - (singleMinute1));       	// just the extra minutes
  if (countMinute == 0)                                     		// change a zero result to be  twelve o clock
    countMinute = 12;

  // second display from here

  countSecond2 = countSecond;                               	 // store count second to compare before next five second change
  if (countSecond2 == 0)                                   	 // adjust if it is zero
    countSecond2 = 12;
  countSecond = (countSecond1 / 5);                         	// convert seconds into fives
  if (countSecond1 == 0)                                   	 // if less than 5 treat as 12
    countSecond = 12;                                      	 // correction for twelve to be read
  if (countSecond <= 11)                                    	// don't flash twelve light
  {
    if (countSecond == countSecond2)                        // if equal begin timer
    {
      secondTimer += 1;                                     	// inc timer
      if (secondTimer == 100)                               	// set to switch off blue led
        singleSecond1 = 0;                                  	// switch off blue led
      if (secondTimer == 480)                               	// set to switch on blue led
        singleSecond1 = 1;                                  	// switch on blue led
    }
    else
    {
      secondTimer = 0;                                      // reset timer
    }
  }
  else
  {
    secondTimer = 0;                                        // make sure timer is reset if it is at twelve
  }

} // end of void display multiplex

//------------ Single minute display -----------------------------

void SingleMinutedisplay()
{
  PORTB = B11000000;                                  	// clear port
  PORTB = barSingleMinute[singleMinute];              // fetch from array
  digitalWrite(7, LOW);                               		// white minute leds
  delayMicroseconds (0.5);                            	// brightness control
  digitalWrite(7, HIGH);                              		// white minute leds
  PORTB = B11000000;                                  	// clear port

  // blue center light for alarm armed
  if (alarmIsOn == 1)                                  		// switch centre alarm light on if  flag  is on
    PORTB = B11000010;                                 	// alarm light on
  else
    PORTB = B11000000;                                 	// Alarm light off
  digitalWrite(7, LOW);                             		 // blue alarm led on
  delayMicroseconds (ledControlFour);                	// brightness control
  digitalWrite(7, HIGH);                            		 // blue alarm led off
  PORTB = B11000000;                                 	// clear port

} // end of void single minute

// ---- minute display ------------------------------------------

void MinuteDisplay()
{
  PORTB = barOneToTwentyFour[countMinute];            // fetch from array
  if (countMinute <= 6)
  {
    digitalWrite(5, LOW);                              		// 1 to 6 drive outer
    delayMicroseconds (ledControlTwo);
    digitalWrite(5, HIGH);
    delay (1);
    digitalWrite(4, LOW);                             		 // 1 to 6 drive inner
    delayMicroseconds (ledControlTwo);
    digitalWrite(4, HIGH);
  }
  else
  {
    digitalWrite(6, LOW);                              		// 7 to 12 drive outer
    delayMicroseconds (ledControlTwo);
    digitalWrite(6, HIGH);
    delay (1);
    digitalWrite(3, LOW);                             		 // 7 to 12 drive inner
    delayMicroseconds (ledControlTwo);
    digitalWrite(3, HIGH);
  }
  PORTB = B11000000;                                   	// clear port

}  // end of void minutes

// ---------- The hour display ------------------------------

void HourDisplay() {

  PORTB = barOneToTwentyFour[countHour];                	// fetch from array
  if (countHour <= 12)                                  			// normal clock display is 1 to 12
  {
    if (countHour <= 6)
    {
      digitalWrite(4, LOW);                            			 // 1 to 6 drive inner
      delayMicroseconds (ledControlTwo);
      digitalWrite(4, HIGH);
    }
    else
    {
      digitalWrite(3, LOW);                            			// 7 to 12 drive inner
      delayMicroseconds (ledControlTwo);
      digitalWrite(3, HIGH);
    }
  }

  //----------------- beyond here is just for setting up the alarm clock and not normally displayed

  else if (countHour <= 18)
  {
    digitalWrite(5, LOW);                              // 13 to 18 drive outer
    delayMicroseconds (ledControlTwo);
    digitalWrite(5, HIGH);
  }
  else
  {
    digitalWrite(6, LOW);                              // 19 to 24 drive outer
    delayMicroseconds (ledControlTwo);
    digitalWrite(6, HIGH);
  }
  PORTB = B11000000;                                   // clear port

} // end of void hour

//------------------------------this is blue lights seconds 1 to 6 --------------------------------

void oneToTwelveBlueLightControl ()
{
  BlueOne = B11111000;                              // default light settings
  BlueTwo = B11000111;
  BlueThree = B11000111;
  BlueFour  = B11111000;

  if (singleSecond1 == 0)                           // this is the flash control
  {
   delayMicroseconds (400); 
    if (countSecond <= 6)
    {
      if (countSecond == 1)
        BlueOne = B11011000;
      else if (countSecond == 2)
        BlueOne = B11101000;
      else if (countSecond == 3)
        BlueOne = B11110000;
      else if (countSecond == 4)
        BlueTwo = B11000011;
      else if (countSecond == 5)
        BlueTwo = B11000101;
      else if (countSecond == 6)
        BlueTwo = B11000110;
    }
    else
    {
      if (countSecond == 7)
        BlueThree = B11000110;
      else if (countSecond == 8)
        BlueThree = B11000101;
      else if (countSecond == 9)
        BlueThree = B11000011;
      else if (countSecond == 10)
        BlueFour = B11110000;
      else if (countSecond == 11)
        BlueFour = B11101000;
      else if (countSecond == 12)
        BlueFour = B11011000;
    }
  }


  PORTB = BlueOne;
  delayMicroseconds (200);
  digitalWrite(A2, LOW);                          		// blue 1 - 3
  delayMicroseconds (ledControlFour);             	// brightness control
  digitalWrite(A2, HIGH);                         		// blue 1 - 3

  PORTB = BlueTwo;
  delayMicroseconds (200);
  digitalWrite(A2, LOW);                          		// blue 4 - 6
  delayMicroseconds (ledControlFour);             	// brightness control
  digitalWrite(A2, HIGH);                        		 // blue 4 - 6

  PORTB = BlueThree;
  delayMicroseconds (200);
  digitalWrite(2, LOW);  // blue 7 to 12        		// blue 7 - 9
  delayMicroseconds (ledControlFour);           	// brightness control
  digitalWrite(2, HIGH);                        		// blue 7 - 9

  PORTB = BlueFour;
  delayMicroseconds (200);
  digitalWrite(2, LOW);                         		// blue 10 - 12
  delayMicroseconds (ledControlFour);           	// brightness control
  digitalWrite(2, HIGH);                        		// blue 10 - 12
  PORTB = B11000000;  // clear port

  PORTB = barSingleMinute[singleMinute];              // fetch from array single minutes
  delayMicroseconds (200);
  digitalWrite(7, LOW);                              		 // white minute leds
  delayMicroseconds (ledControlThree);                	 // brightness control
  digitalWrite(7, HIGH);                             		 // white minute leds
  PORTB = B11000000;                                  	// clear port

  if (alarmIsOn == 1)                                 		 // switch centre alarm light on if  flag  is on
    PORTB = B11000010;                                 	// alarm light on
  else
    PORTB = B11000000;                                 	// Alarm light off
  digitalWrite(7, LOW);                              		// blue alarm led on
  delayMicroseconds (ledControlFour);                	// brightness control
  digitalWrite(7, HIGH);                             		// blue alarm led off
  PORTB = B11000000;                                 	// clear port


} // end of void for outer lights

//_____________________________ brightness control _________________________________

void BrightnessControl () {
  ledControlOne += 1;

  if (ledControlOne >= 6)
    ledControlOne = 1;
  ledControlTwo = barBrightnessControl[ledControlOne];
  ledControlThree = whiteBrightnessControl[ledControlOne];
  ledControlFour = outerBrightnessControl[ledControlOne];

  EEPROM.write(4, (ledControlOne));           	// store result for power down
  buttonTest = digitalRead (A0);             		// left centre button for dim lights
  while (buttonTest == 0)                    		// if button is still pressed wait till release
  {
    delay (10);
    buttonTest = digitalRead (A0);             		// wait for button release
    delay (10);
  }
  digitalWrite(A3, HIGH);   			//  button release bleep on
  delay (5);
  digitalWrite(A3, LOW);    			//  button release bleep off

}  // end of void for brightness control